﻿-- Author      : Anthony White
-- Create Date : 1/15/2008 3:19:37 PM
-- FuBar_AlarmClockFu 1.0.5
-- A simple alarm clock module for FuBar users

-- Setup libs and Rock
local Tablet = AceLibrary("Tablet-2.0")

AlarmClockFu = Rock:NewAddon("AlarmClockFu", "LibRockTimer-1.0", "LibRockDB-1.0", "LibRockConfig-1.0", "LibFuBarPlugin-3.0", "LibRockConsole-1.0")

-- FuBar Plugin Info
AlarmClockFu.version = "1.0.0"
AlarmClockFu:SetFuBarOption("hasIcon", false)
AlarmClockFu:SetFuBarOption("defaultPosition", "RIGHT")
AlarmClockFu:SetFuBarOption("tooltipType", "Tablet-2.0")

-- SavedVariables Table (Set default options)
AlarmClockFu:SetDatabase("AlarmClockFuDB")

AlarmClockFu:SetDatabaseDefaults('profile', {
	twentyFour = false, -- Twenty four / Twelve hour time
	bothTimes = false, -- Display Local Time | Server Time
	localTime = true, -- Display Local Time OR Server Time
	alarm1 = { hour = 0, minute = 0, message = "Alarm 1 Activated!", enabled = false, am = true }, -- Set up the alarms
	alarm2 = { hour = 0, minute = 0, message = "Alarm 2 Activated!", enabled = false, am = true }, -- ...
	alarm3 = { hour = 0, minute = 0, message = "Alarm 3 Activated!", enabled = false, am = true }, -- ...
})
	
local ALARM_ONE = 1
local ALARM_TWO = 2
local ALARM_THREE = 3

local lastAlarmMinute = nil

-- OnEnable
function AlarmClockFu:OnEnable()
	-- make sure we update the plugin once a second (for hopefully accurate times)
	self:AddRepeatingTimer(1, "UpdateFuBarPlugin")
end

-- OnDisable
function AlarmClockFu:OnDisable()
	-- empty
	-- having the function defined prevents errors when users try to disable the plugin
end

-- Get / Set Methods

--[[ 12 / 24 Hour Format ]]--

function AlarmClockFu:IsTwentyFour()
	return self.db.profile.twentyFour
end

function AlarmClockFu:ToggleTwentyFour()
	self.db.profile.twentyFour = not self.db.profile.twentyFour
	self:UpdateFuBarPlugin()
	return self.db.profile.twentyFour
end

--[[ Local / Server / Both Time Display ]]--

function AlarmClockFu:IsLocalTime()
	return self.db.profile.localTime
end

function AlarmClockFu:ToggleLocalTime()
	self.db.profile.localTime = not self.db.profile.localTime
	self:UpdateFuBarPlugin()
end

function AlarmClockFu:IsBothTimes()
	return self.db.profile.bothTimes
end

function AlarmClockFu:ToggleBothTimes()
	self.db.profile.bothTimes = not self.db.profile.bothTimes
	self:UpdateFuBarPlugin()
end

-- END GET / SET METHODS --

function AlarmClockFu:OnInitialize()
	-- Setup config table
	local config = {
		name = "FuBar_AlarmClockFu",
		desc = self.notes,
		type = 'group',
		args = {
			display = {
				type = 'group',
				name = "Display Settings",
				desc = "Choose how the time will be displayed",
				args = {
					disp_local = {
						type = 'toggle',
						name = "Local Time",
						desc = "Toggle between displaying local or server time",
						get = "IsLocalTime",
						set = "ToggleLocalTime"
					},
					disp_both = {
						type = 'toggle',
						name = "Show Both",
						desc = "Display both times",
						get = "IsBothTimes",
						set = "ToggleBothTimes"
					},
					twentyFour = {
						type = 'toggle',
						name = "24 Hour Format",
						desc = "Toggle between 12 and 24 hour format",
						get = "IsTwentyFour",
						set = "ToggleTwentyFour"
					}
				}
			},
			alarm1 = {
				type = "group",
				name = "Alarm 1 Settings",
				desc = "Configure Alarm 1",
				args = {
					hour = {
						order = 1,
						type = "text",
						name = "Hour",
						desc = "Set the alarm hour (1 to 12)",
						usage = "<hour>",
						get = function()
							return AlarmClockFu.db.profile.alarm1.hour
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm1.hour = v
						end,
					},
					minute = {
						order = 2,
						type = "text",
						name = "Minute",
						desc = "Set the alarm minute (0 to 59)",
						usage = "<minute>",
						get = function()
							return AlarmClockFu.db.profile.alarm1.minute
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm1.minute = v
						end,
					},
					enabled = {
						order = 0,
						type = "toggle",
						name = "Enable Alarm",
						desc = "Toggle the alarm on or off",
						get = function()
							return AlarmClockFu.db.profile.alarm1.enabled
						end,
						set = function()
							AlarmClockFu.db.profile.alarm1.enabled = not AlarmClockFu.db.profile.alarm1.enabled
						end
					},
					am = {
						type = "toggle",
						name = "AM",
						desc = "Given time is in the AM",
						get = function()
							return AlarmClockFu.db.profile.alarm1.am
						end,
						set = function()
							AlarmClockFu.db.profile.alarm1.am = true
						end,
					},
					pm = {
						type = "toggle",
						name = "PM",
						desc = "Given time is in the PM",
						get = function()
							return not AlarmClockFu.db.profile.alarm1.am
						end,
						set = function()
							AlarmClockFu.db.profile.alarm1.am = false
						end,
					},
					message = {
						order = 3,
						type = "text",
						name = "Message",
						desc = "Alarm 1 Message",
						usage = "<message, string>",
						get = function()
							return AlarmClockFu.db.profile.alarm1.message
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm1.message = v
						end
					}
				}
			},
			alarm2 = {
				type = "group",
				name = "Alarm 2 Settings",
				desc = "Configure Alarm 2",
				args = {
					hour = {
						order = 1,
						type = "text",
						name = "Hour",
						desc = "Set the alarm hour (1 to 12)",
						usage = "<hour>",
						get = function()
							return AlarmClockFu.db.profile.alarm2.hour
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm2.hour = v
						end,
					},
					minute = {
						order = 2,
						type = "text",
						name = "Minute",
						desc = "Set the alarm minute (0 to 59)",
						usage = "<minute>",
						get = function()
							return AlarmClockFu.db.profile.alarm2.minute
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm2.minute = v
						end,
					},
					enabled = {
						order = 0,
						type = "toggle",
						name = "Enable Alarm",
						desc = "Toggle the alarm on or off",
						get = function()
							return AlarmClockFu.db.profile.alarm2.enabled
						end,
						set = function()
							AlarmClockFu.db.profile.alarm2.enabled = not AlarmClockFu.db.profile.alarm2.enabled
						end
					},
					am = {
						type = "toggle",
						name = "AM",
						desc = "Given time is in the AM",
						get = function()
							return AlarmClockFu.db.profile.alarm2.am
						end,
						set = function()
							AlarmClockFu.db.profile.alarm2.am = true
						end,
					},
					pm = {
						type = "toggle",
						name = "PM",
						desc = "Given time is in the PM",
						get = function()
							return not AlarmClockFu.db.profile.alarm2.am
						end,
						set = function()
							AlarmClockFu.db.profile.alarm2.am = false
						end,
					},
					message = {
						order = 3,
						type = "text",
						name = "Message",
						desc = "Alarm 2 Message",
						usage = "<message, string>",
						get = function()
							return AlarmClockFu.db.profile.alarm2.message
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm2.message = v
						end
					}
				}
			},
			alarm3 = {
				type = "group",
				name = "Alarm 3 Settings",
				desc = "Configure Alarm 3",
				args = {
					hour = {
						order = 1,
						type = "text",
						name = "Hour",
						desc = "Set the alarm hour (1 to 12)",
						usage = "<hour>",
						get = function()
							return AlarmClockFu.db.profile.alarm3.hour
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm3.hour = v
						end,
					},
					minute = {
						order = 2,
						type = "text",
						name = "Minute",
						desc = "Set the alarm minute (0 to 59)",
						usage = "<minute>",
						get = function()
							return AlarmClockFu.db.profile.alarm3.minute
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm3.minute = v
						end,
					},
					enabled = {
						order = 0,
						type = "toggle",
						name = "Enable Alarm",
						desc = "Toggle the alarm on or off",
						get = function()
							return AlarmClockFu.db.profile.alarm3.enabled
						end,
						set = function()
							AlarmClockFu.db.profile.alarm3.enabled = not AlarmClockFu.db.profile.alarm3.enabled
						end
					},
					am = {
						type = "toggle",
						name = "AM",
						desc = "Given time is in the AM",
						get = function()
							return AlarmClockFu.db.profile.alarm3.am
						end,
						set = function()
							AlarmClockFu.db.profile.alarm3.am = true
						end,
					},
					pm = {
						type = "toggle",
						name = "PM",
						desc = "Given time is in the PM",
						get = function()
							return not AlarmClockFu.db.profile.alarm3.am
						end,
						set = function()
							AlarmClockFu.db.profile.alarm3.am = false
						end,
					},
					message = {
						order = 3,
						type = "text",
						name = "Message",
						desc = "Alarm 3 Message",
						usage = "<message, string>",
						get = function()
							return AlarmClockFu.db.profile.alarm3.message
						end,
						set = function(v)
							AlarmClockFu.db.profile.alarm3.message = v
						end
					}
				}
			},			
		}
	}
	
	AlarmClockFu:SetConfigTable(config)
	AlarmClockFu.OnMenuRequest = config
	
	-- set up alarm dialog boxes
	StaticPopupDialogs["ALARMCLOCKFU_ALARM1"] = {
		text = AlarmClockFu.db.profile.alarm1.message,
		button1 = "OK",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	}

	StaticPopupDialogs["ALARMCLOCKFU_ALARM2"] = {
		text = AlarmClockFu.db.profile.alarm2.message,
		button1 = "OK",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	}

	StaticPopupDialogs["ALARMCLOCKFU_ALARM3"] = {
		text = AlarmClockFu.db.profile.alarm3.message,
		button1 = "OK",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	}
end

-- Format the given hour / minute into a string to be displayed
function AlarmClockFu:FormatTime(hour, minute)
	if self:IsTwentyFour() then
		-- we want to display a twenty four hour time
		return string.format("|cffffffff%02d|r:|cffffffff%02d|r", hour, minute)
	else
		-- display 12 hour format
		hour = mod(hour, 12)
		if hour == 0 then
			hour = 12
		end
		
		return string.format("|cffffffff%02d|r:|cffffffff%02d|r %s", hour, minute, date("%p"))
	end
end

-- Get the current local time
-- Should be accurate to the second
function AlarmClockFu:GetLocalTime()
	local hour = tonumber(date("%H"))
	local minute = tonumber(date("%M"))
	return self:FormatTime(hour, minute), hour, minute
end

-- Get the current server time
-- This tends to be inaccurate as to the current minute
function AlarmClockFu:GetServerTime()
	local hour, minute = GetGameTime()
	return self:FormatTime(hour, minute), hour, minute
end

-- Alarm helper method (converts the hours into 12 / 24 hour depending on user settings)
function AlarmClockFu:CalculateAlarmHours()
	-- set up a temp table
	local alarms = {
		alarm1 = {
			hour = self.db.profile.alarm1.hour,
			minute = self.db.profile.alarm1.minute,
			enabled = self.db.profile.alarm1.enabled
		},
		alarm2 = {
			hour = self.db.profile.alarm2.hour,
			minute = self.db.profile.alarm2.minute,
			enabled = self.db.profile.alarm2.enabled
		},
		alarm3 = {
			hour = self.db.profile.alarm3.hour,
			minute = self.db.profile.alarm3.minute,
			enabled = self.db.profile.alarm3.enabled
		}
	}
	
	if self:IsTwentyFour() then
		if self.db.profile.alarm1.am == false then
			-- we need to offset the alarm hour by 12
			alarms.alarm1.hour = alarms.alarm1.hour + 12
		end
		
		if self.db.profile.alarm2.am == false then
			alarms.alarm2.hour = alarms.alarm2.hour + 12
		end
		
		if self.db.profile.alarm3.am == false then
			alarms.alarm3.hour = alarms.alarm3.hour + 12
		end
	end
	
	return alarms
end

-- Alarm helper method (checks to see if an alarm is ready)
function AlarmClockFu:IsAlarmReady(alarm)
	local _, hour, minute = self:GetLocalTime()

	local alarms = self:CalculateAlarmHours()

	if alarm == ALARM_ONE then
		return ((self:FormatTime(alarms.alarm1.hour, alarms.alarm1.minute) == self:GetLocalTime()) and (lastAlarmMinute ~= minute) and alarms.alarm1.enabled)
	elseif alarm == ALARM_TWO then
		return ((self:FormatTime(alarms.alarm2.hour, alarms.alarm2.minute) == self:GetLocalTime()) and (lastAlarmMinute ~= minute) and alarms.alarm2.enabled)
	else
		return ((self:FormatTime(alarms.alarm3.hour, alarms.alarm3.minute) == self:GetLocalTime()) and (lastAlarmMinute ~= minute) and alarms.alarm3.enabled)
	end
end

-- Called once per second via the timer set in AlarmClockFu:OnEnable
-- Sets the FuBar text to the current time
function AlarmClockFu:UpdateFuBarText()
	local str = ""
	
	local _, hour, minute = self:GetLocalTime()
	
	if self:IsAlarmReady(ALARM_ONE) then
		-- alarm 1 needs to be triggered
		StaticPopup_Show("ALARMCLOCKFU_ALARM1")
		PlaySound("ReadyCheck")
		
		lastAlarmMinute = minute
	elseif self:IsAlarmReady(ALARM_TWO) then
		StaticPopup_Show("ALARMCLOCKFU_ALARM2")
		PlaySound("ReadyCheck")
		
		lastAlarmMinute = minute
	elseif self:IsAlarmReady(ALARM_THREE) then
		StaticPopup_Show("ALARMCLOCKFU_ALARM3")
		PlaySound("ReadyCheck")
		
		lastAlarmMinute = minute
	end

	if self:IsBothTimes() then
		if self:IsLocalTime() then
			str = string.format("%s || %s", self:GetLocalTime(), self:GetServerTime())
		else
			str = string.format("%s || %s", self:GetServerTime(), self:GetLocalTime())
		end
	else
		if self:IsLocalTime() then
			str = self:GetLocalTime()
		else
			str = self:GetServerTime()
		end
	end
	
	self:SetFuBarText(str)
end

-- FuBar Tooltip Display
function AlarmClockFu:OnUpdateFuBarTooltip()
	-- insert the current date
	Tablet:AddCategory():AddLine(
		'text', date("%A, %B %d, %Y"),
		'textR', 1,
		'textG', 1,
		'textB', 1,
		'justify', "CENTER"
	)

	local cat = Tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	
	cat:AddLine(
		'text', "Local Time",
		'text2', self:GetLocalTime()
	)
	
	cat:AddLine(
		'text', "Server Time",
		'text2', self:GetServerTime()
	)
end